package de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation;

import java.util.Collection;
import java.util.Iterator;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.Controller;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.NetworkManipulationAlgorithm;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;

/**
 * Simple proof of concept NetworkManipulationAlgorithm, which moves one random visible SmartDevice by a small amount.
 *
 * @author Andreas T. Meyer-Berg
 */
public class SimpleManipulation implements NetworkManipulationAlgorithm {

	@Override
	public void runAlgorithm(Controller controller, long currentTimeStep) {
		/**
		 * Visible devices of the model, to select one which should be moved
		 */
		Collection<SmartDevice> devices = controller.getNetworkController().getVisibleSmartDevices();
		/**
		 * Iterator for the devices
		 */
		Iterator<SmartDevice> it = devices.iterator();
		/**
		 * SmartDevice which should be moved
		 */
		SmartDevice move = null;
		/**
		 * Index of the randomly selected device
		 */
		int deviceNumber = (int)Math.round(devices.size() * Math.random());
		for(int i = 0; i< deviceNumber && it.hasNext(); i++){
			move = it.next();
		}
		/**
		 * Move device randomly up to 5 pixels in x and y direction
		 */
		if(move != null){
			controller.getNetworkController().moveSmartDevice(move,(int)(move.getX()+Math.random()*10-5), (int)(move.getY()+Math.random()*10-5), move.getZ());
			controller.getNetworkController().validateDevicePosition();
		}
			/**
		 * Refresh Panel
		 */
		controller.notifyObservers();
	}

}
